package cz.cvut.fit.xml.index.tpa;

import java.io.Serializable;
import java.util.Objects;
import java.util.TreeSet;

/**
 * State of a determnistic finite automaton.
 *
 * @author sestaeli
 */
class State implements Serializable {

    private final TreeSet<Integer> dsubset;

    /**
     * Constructs new <code>State</code> with empty d-subset.
     */
    public State() {
        dsubset = new TreeSet<>();
    }

    /**
     * Constructs new <code>State</code> with d-subset containing id.
     *
     * @param id identifier of nondetermnistic state
     */
    public State(int id) {
        dsubset = new TreeSet<>();
        dsubset.add(id);
    }

    @Override
    public String toString() {
        StringBuilder s = new StringBuilder("[");
        dsubset.forEach((id) -> {
            s.append(id).append(",");
        });
        s.deleteCharAt(s.lastIndexOf(","));
        s.append("]");
        return s.toString();
    }

    /**
     * Update d-subset of the state by adding given state if not already
     * present.
     *
     * @param id identifier of nondeterministic state to be added into d-subset
     */
    public void updateDSubset(int id) {
        dsubset.add(id);
    }

    /**
     * Update d-subset of the state by adding given states if not already
     * present.
     *
     * @param subset subset of ids of nondetrministic states to be added into
     * d-subset
     */
    public void updateDSubset(TreeSet<Integer> subset) {
        dsubset.addAll(subset);
    }

    /**
     * Get the set of states included in d-subset.
     *
     * @return set of states in d-subset
     */
    public TreeSet<Integer> getDSubset() {
        return dsubset;
    }

    @Override
    public int hashCode() {
        int hash = 3;
        hash = 37 * hash + Objects.hashCode(this.dsubset);
        return hash;
    }

    @Override
    public boolean equals(Object obj) {
        if (obj == null) {
            return false;
        }
        if (getClass() != obj.getClass()) {
            return false;
        }
        final State other = (State) obj;
        if (!Objects.equals(this.dsubset, other.dsubset)) {
            return false;
        }
        return true;
    }
}
